### Scope作用域
在 Spring IoC 容器中具有以下几种作用域：

- singleton：单例模式，在整个Spring IoC容器中，使用singleton定义的Bean将只有一个实例，适用于无状态bean；
- prototype：原型模式，每一次请求（将其注入到另一个bean中，或者以程序的方式调用容器的getBean()方法）都会产生一个新的bean实例，相当于一个new的操作；
- request：对于每次HTTP请求，使用request定义的Bean都将产生一个新实例，即每次HTTP请求将会产生不同的Bean实例。只有在Web应用中使用Spring时，该作用域才有效；
- session：对于每次HTTP Session，使用session定义的Bean豆浆产生一个新实例。同样只有在Web应用中使用Spring时，该作用域才有效；
- globalsession：每个全局的HTTP Session，使用session定义的Bean都将产生一个新实例。典型情况下，仅在使用portlet context的时候有效。同样只有在Web应用中使用Spring时，该作用域才有效。

@scope默认是单例模式（singleton）,如果需要设置的话@scope("prototype")

### 无状态会话bean
bean一旦实例化就被加进会话池中，各个用户都可以共用。即使用户已经消亡，bean 的生命期也不一定结束，它可能依然存在于会话池中，供其他用户调用。
由于没有特定的用户，那么也就不能保持某一用户的状态，所以叫无状态bean。但无状态会话bean 并非没有状态，如果它有自己的属性（变量），那么这些变量就会受到所有调用它的用户的影响。

### 有状态会话bean
每个用户有自己特有的一个实例，在用户的生存期内，bean保持了用户的信息，即“有状态”；一旦用户灭亡（调用结束或实例结束），bean的生命期也告结束。即每个用户最初都会得到一个初始的bean。

有状态bean，如果配置为singleton，会出现线程安全问题。可以参见```ThreadError```类例子分析

### 在Servlet中
Servlet体系结构是建立在Java多线程机制之上的，它的生命周期是由Web 容器负责的。
一个Servlet类在Application中只有一个实例存在，有多个线程在使用这个实例。这是单例模式的应用。
无状态的单例是线程安全的，但我们如果在Servlet里用了实例变量(私有变量)，那么就变成有状态了，是非线程安全的。
如下面的用法就是不安全的,因为user是有状态信息的。
```java
public class UserServlet HttpServlet{     
    private User user;   
    public void doGet (HttpServletRequest request, HttpServletResponse response)throws ServletException, IOException{  
        //do something...  
    }  
} 
```

### 总结
- singleton会造成资源混乱问题,而如果是prototype的话，就不会出现资源共享的问题。（即不会出现线程安全的问题）
- 应该尽量使用无状态Bean.如果在程序中出现私有变量（该bean会变为有状态的，一旦在其他线程中发生改变，就会产生线程不安全），解决方案就是尽量替换为方法中的参数。对于每个访问私有变量的方法增加变量传入（参数传入）或者通过ThreadLocal来获取。
- 如果用有状态的bean，就要用prototype模式，每次在注入的时候就重新创建一个bean，在多线程中互不影响。
- 如Service层、Dao层用默认singleton就行，虽然Service类也有dao这样的属性，但dao这些类都是没有状态信息的，也就是相当于不变(immutable)类，所以不影响。
- Stateless无状态用单例Singleton模式，Stateful有状态就用原型Prototype模式。